static const char *RcsId = "$Header: /home/cvsadm/cvsroot/fermi/servers/guntiming/src/Guntiming.cpp,v 1.2 2020-03-11 11:21:11 giulio Exp $";
//+=============================================================================
//
// file :	 Guntiming.cpp
//
// description :  C++ source for the Guntiming and its commands.
//		The class is derived from Device. It represents the
//		CORBA servant object which will be accessed from the
//		network. All commands which can be executed on the
//		Guntiming are implemented in this file.
//
// project :      TANGO Device Server
//
// $Author: giulio $
//
// $Revision: 1.2 $
//
// $Log: Guntiming.cpp,v $
// Revision 1.2  2020-03-11 11:21:11  giulio
// Moved from PPC to x64
//
// Revision 1.1.1.1  2014-12-11 14:05:08  zambonl
// Prima versione
//
// Revision 1.1.1.1  2014-11-11 10:00:36  zambonl
// Prima versione
//
// Revision 1.1.1.1  2013-08-06 08:24:53  zambonl
// Prima versione
//
//
// copyleft :     European Synchrotron Radiation Facility
//		BP 220, Grenoble 38043
//		FRANCE
//
//-=============================================================================
//
//	      This file is generated by POGO
//      (Program Obviously used to Generate tango Object)
//
//	 (c) - Software Engineering Group - ESRF
//=============================================================================



//===================================================================
//
//      The following table gives the correspondance
//      between commands and method's name.
//
//  Command's name|  Method's name
//      ----------------------------------------
//  State      |  dev_state()
//  Status     |  dev_status()
//  Raw        |  raw()
//  Stinit     |  stinit()
//  Restart    |  restart()
//
//===================================================================

/*
help
HW v2.0.1 FW v0.1b 08 Dic 2024
Modify on 08 DIC 2024 PIC 24FJ256GA110
Valid command |RD_STAT|RD_INFO|HELP|ST_MODE,0,1,2|ST_NB,n|ST_DACB-C,|ST_GRID,word|ST_V80,word|BY_xxxx|ST_IF,0-1|ST_LD,0-1|STOP
ST_DRV,0-15|ST_INIT|ST_CUR,word|ST_PARA,KV80-KGRID-KVDRIVER-KIDRIVER,float|ST_FLASH

RD_STAT
DAC:0,2000,1800,2500;ADC;V pol:78.15682;V Grid:1.25000;VDriver:0.00000;IDriver:0.00000;I mA filament set:1700;I mA filament read:1701;mV filament:1758;Flags:1;Temperature: 29.8;NB:  120;MODE:    0;Serial number:    0

RD_INFO
HW v2.0.1 FW v0.1d 05 Feb 2025
Created on 08 DIC 2024 PIC 24FJ256GA110
Serial number=    0 KV=7.85599 KVGRID=7.19999 KVDRIVER=1.00000 KIDRIVER=1.57000 V80M= 2500 V80S= 2500 V80M= 2500 V80S= 2500 DACC MB= 1800 DACC SB= 1500
GUN  Filament OK
Interlock Y
Interrupt received from pin 18 =  0
YES i am alive , all OK UART speed 57600

*/

#include <tango.h>
#include <Guntiming.h>
#include <GuntimingClass.h>

#include <stdio.h>
#include <sys/socket.h>
#include <arpa/inet.h>
#include <unistd.h>
#include <string.h>
#include <string>
#include <stdexcept>
#include <iostream>
#include <time.h>

char buf[2000];
char tstr[80];
class Socket {
	int sock;
public:
	Socket(int socket) : sock(socket) {
		if (sock<0) throw std::runtime_error("Socket creation error");
	}
	Socket() : Socket(socket(AF_INET, SOCK_STREAM, 0)) {}
	std::string rx() {
		char buffer[1024] = {0};
		int n = read( sock , buffer, sizeof(buffer));
		return std::string(buffer,n);
	}
	void tx(std::string s) {
		send(sock , s.c_str() , s.length() , 0);
	}
	int getSocket() {
		printf("\nsock %d\n", sock);
		return sock;
	}
};

string linedelimiter = ";"; // "\r";
std::vector<std::string> tokens;
std::string srx;
Socket sc;
int myconnected = -1;
void myconnect(int socket, std::string address, unsigned short port) {
	struct sockaddr_in serv_addr;
	serv_addr.sin_family = AF_INET;
	serv_addr.sin_port = htons(port);
	// Convert IPv4 and IPv6 addresses from text to binary form
	if(inet_pton(AF_INET, address.c_str(), &serv_addr.sin_addr) <= 0) throw std::runtime_error("Invalid address: Address not supported");
	myconnected = connect(socket, (struct sockaddr *)&serv_addr, sizeof(serv_addr));
	if (myconnected < 0) throw std::runtime_error("\nConnection Failed \n");
}

namespace Guntiming_ns
{

bool scinited = false;

//+----------------------------------------------------------------------------
//
// method :	     Guntiming::Guntiming(string &s)
//
// description :	constructor for simulated Guntiming
//
// in : - cl : Pointer to the DeviceClass object
//      - s : Device name
//
//-----------------------------------------------------------------------------
Guntiming::Guntiming(Tango::DeviceClass *cl,string &s)
:TANGO_BASE_CLASS(cl,s.c_str())
{
	init_device();
}

Guntiming::Guntiming(Tango::DeviceClass *cl,const char *s)
:TANGO_BASE_CLASS(cl,s)
{
	init_device();
}

Guntiming::Guntiming(Tango::DeviceClass *cl,const char *s,const char *d)
:TANGO_BASE_CLASS(cl,s,d)
{
	init_device();
}
/*
  timestamp()
  send a human readable timestamp to standard output
*/
void Guntiming::timestamp()
{
	struct tm *mytime;
	time_t myTime_t;
	myTime_t = time(NULL);
	mytime = localtime(&myTime_t);
	strftime(tmBuffer, 79, "%Y-%m-%d %H:%M:%S ", mytime);
	return;
}
//+----------------------------------------------------------------------------
//
// method :	     Guntiming::delete_device()
//
// description :	will be called at device destruction or at init command.
//
//-----------------------------------------------------------------------------
void Guntiming::delete_device()
{
	//      Delete device's allocated object
	delete[] attr_FilCur_read;
	delete[] attr_FilCurset_read;
	delete[] attr_FilVoltage_read;
	delete[] attr_PolVoltage_read;
	delete[] attr_Temperature_read;
	delete[] attr_Mode_read;
	delete[] attr_Modestring_read;
	delete[] attr_DriverCur_read;
	delete[] attr_DriverVoltage_read;
	delete[] attr_GridVoltage_read;
	delete[] attr_BunchLength_read;
	delete[] attr_StateFlags_read;

}

//+----------------------------------------------------------------------------
//
// method :	     Guntiming::init_device()
//
// description :	will be called at device initialization.
//
//-----------------------------------------------------------------------------
void Guntiming::init_device()
{
	INFO_STREAM << "Guntiming::Guntiming() create device " << device_name << endl;

	// Initialise variables to default values
	//--------------------------------------------
	get_device_property();

	attr_FilCur_read = new Tango::DevDouble[1];
	attr_FilCurset_read = new Tango::DevDouble[1];
	attr_FilVoltage_read = new Tango::DevDouble[1];
	attr_PolVoltage_read = new Tango::DevDouble[1];
	attr_Temperature_read = new Tango::DevDouble[1];
	attr_Mode_read = new Tango::DevDouble[1];
	attr_Modestring_read = new Tango::DevString;
	attr_DriverCur_read = new Tango::DevDouble[1];
	attr_DriverVoltage_read = new Tango::DevDouble[1];
	attr_GridVoltage_read = new Tango::DevDouble[1];
	attr_BunchLength_read = new Tango::DevDouble[1];
	attr_StateFlags_read = new Tango::DevBoolean[1];

	// alloc array used for status bits
	datab.length(6);

	if (myconnected >= 0) return;
	try {
		cout << "connecting IP: " << IP << endl;
		myconnect(sc.getSocket(), IP, 10001);
		sc.tx("RD_STAT\n");
		// sleep 100 ms
		nanosleep((const struct timespec[]){{0, 100000000L}}, NULL);
		srx = sc.rx();
		cout << srx << endl;
		size_t pos = 0;
		std::string token;
		while ((pos = srx.find(linedelimiter)) != std::string::npos) {
			token = srx.substr(0, pos);
			cout << "token: " << token << endl;
        		tokens.push_back(token);
        		srx.erase(0, pos + linedelimiter.length());
    		}
		tokens.push_back(srx);
		tokens.push_back("");
		tokens.push_back("");
	} catch (exception &e) {
		timestamp(); cerr << tmBuffer << "sc.getSocket() " << sc.getSocket() << endl;
		cerr << e.what() << endl;
	}
}


//+----------------------------------------------------------------------------
//
// method :	     Guntiming::get_device_property()
//
// description :	Read the device properties from database.
//
//-----------------------------------------------------------------------------
void Guntiming::get_device_property()
{
	//      Initialize your default values here (if not done with  POGO).
	//------------------------------------------------------------------

	//      Read device properties from database.(Automatic code generation)
	//------------------------------------------------------------------
	Tango::DbData   dev_prop;
	dev_prop.push_back(Tango::DbDatum("IP"));

	//      Call database and extract values
	//--------------------------------------------
	if (Tango::Util::instance()->_UseDb==true)
		get_db_device()->get_property(dev_prop);
	Tango::DbDatum  def_prop, cl_prop;
	GuntimingClass  *ds_class =
		(static_cast<GuntimingClass *>(get_device_class()));
	int     i = -1;

	//      Try to initialize IP from class property
	cl_prop = ds_class->get_class_property(dev_prop[++i].name);
	if (cl_prop.is_empty()==false)  cl_prop  >>  IP;
	//      Try to initialize IP from default device value
	def_prop = ds_class->get_default_device_property(dev_prop[i].name);
	if (def_prop.is_empty()==false) def_prop  >>  IP;
	//      And try to extract IP value from database
	if (dev_prop[i].is_empty()==false)      dev_prop[i]  >>  IP;

	//      End of Automatic code generation
	//------------------------------------------------------------------

}


//+------------------------------------------------------------------
/**
 *      method: Psc::set_DevStat
 *
 *      Convert Tango DevState DevStatus according to PsStat boolean array
 */
//+------------------------------------------------------------------
void Guntiming::set_DevStat(void)
{
	/*try {
		devoptpdelay->read_attribute("CounterEnabledMask") >> optpdelay_status;
		timestamp(); cout << tmBuffer << " optpdelay_status" << optpdelay_status << endl;
	}
	catch(Tango::DevFailed &e) {
		TangoSys_MemStream out_stream;
		out_stream << "Error reading timing board" << ends;
		Tango::Except::re_throw_exception(e,
			(const char *)"Reading timing pulses failed",
			out_stream.str(),
			(const char *) "Guntiming::set_DevStat()", Tango::ERR);
	}
	catch(...) {
	}
	gettimeofday(&last_state_time,NULL);
	*/
}


//+----------------------------------------------------------------------------
//
// method :	     Guntiming::always_executed_hook()
//
// description :	method always executed before any command is executed
//
//-----------------------------------------------------------------------------
void Guntiming::always_executed_hook()
{
	struct timeval curr_state_time;
	if (myconnected < 0) {set_state(Tango::FAULT); return;}
	gettimeofday(&curr_state_time, NULL);
	// do not read too frequently status
	if (abs((curr_state_time.tv_sec - last_state_time.tv_sec)*1000000+(curr_state_time.tv_usec - last_state_time.tv_usec)) > GUNTIMING_STATE_TOUT) {
		try {
			sc.tx("RD_STAT\n");
			// sleep 100 ms
			nanosleep((const struct timespec[]){{0, 100000000L}}, NULL);
			srx = sc.rx();
			// cout << srx << endl;
			size_t pos = 0;
			std::string token;
			int i=0;
			while ((pos = srx.find(linedelimiter)) != std::string::npos) {
				token = srx.substr(0, pos);
				cout << "tokens[" << i << "]: " << token << endl;
        			if (tokens.size()>i) tokens[i++] = token; else tokens.push_back(token);
        			srx.erase(0, pos + linedelimiter.length());
    			}
			if (tokens.size()>i) tokens[i++] = srx; else tokens.push_back(token);
			set_state(Tango::ON);
			// set_state(tokens[6].find("YES i am alive , all OK UART speed 57600") != std::string::npos? Tango::ON: Tango::FAULT);
			set_status("Connected to Lantronix board");
		}
		catch (...) {
			std::exception_ptr p = std::current_exception();
			timestamp(); cerr << tmBuffer << "ERROR RD_STAT " <<(p ? p.__cxa_exception_type()->name() : "null") << std::endl;
			set_state(Tango::FAULT);
			set_status("Failed to get gun timing status");
			myconnected = -1;
		}
		gettimeofday(&last_state_time,NULL);
	}
}

//+----------------------------------------------------------------------------
//
// method :	     Guntiming::read_attr_hardware
//
// description :	Hardware acquisition for attributes.
//
//-----------------------------------------------------------------------------
void Guntiming::read_attr_hardware(vector<long> &attr_list)
{
	DEBUG_STREAM << "Guntiming::read_attr_hardware(vector<long> &attr_list) entering... "<< endl;
	//      Add your own code here
}
//+----------------------------------------------------------------------------
//
// method :	     Guntiming::read_StateFlags
//
// description :	Extract real attribute values for StateFlags acquisition result.
//
//-----------------------------------------------------------------------------
void Guntiming::read_StateFlags(Tango::Attribute &attr)
{
	DEBUG_STREAM << "Guntiming::read_StateFlags(Tango::Attribute &attr) entering... "<< endl;
	int numflags = 4;
	if (tokens.size()<10) {
		for (int i=0; i<numflags; i++) {
			attr_StateFlags_read[i] = false;
		}
	}
	else {
		int flags = std::stoi(tokens[9].substr(tokens[9].find(":") + 1));
		for (int i=0; i<numflags; i++) {
			attr_StateFlags_read[i] = ((flags >> i) & 1) == 1;
		}
	}
	attr_StateFlags_read[numflags] = myconnected >= 0;
	attr.set_value(attr_StateFlags_read, numflags+1, 0, false);
}

//+----------------------------------------------------------------------------
//
// method :	     Guntiming::write_GridVoltage
//
// description :	Write GridVoltage attribute values to hardware.
//
//-----------------------------------------------------------------------------
void Guntiming::write_GridVoltage(Tango::WAttribute &attr)
{
	DEBUG_STREAM << "Guntiming::write_GridVoltage(Tango::WAttribute &attr) entering... "<< endl;
	if (myconnected < 0) {Tango::Except::throw_exception((const char *) "NOT CONNECTED",(const char *) "NOT CONNECTED",(const char *) "Guntiming::write_FilCur()", Tango::ERR);}
	attr.get_write_value(attr_GridVoltage_write);
	char cmd[200];
	sprintf(cmd, "ST_GRID,%3.0f\n", attr_GridVoltage_write);
	cout << cmd << endl;
	sc.tx(cmd);
	nanosleep((const struct timespec[]){{1, 500000000L}}, NULL);
	srx = sc.rx();
	cout << "write_GridVoltage(): " << srx << endl;
	if (srx.find("SET Grid =") == std::string::npos) {
		Tango::Except::throw_exception(
			(const char *) cmd,
			(const char *) srx.c_str(),
			(const char *) "Guntiming::write_FilCur()", Tango::ERR);
		timestamp(); cerr << tmBuffer << "Guntiming::write_FilCur() " << srx.c_str() << endl;
	}
}

//+----------------------------------------------------------------------------
//
// method :	     Guntiming::write_Mode
//
// description :	Write Mode attribute values to hardware.
//
//-----------------------------------------------------------------------------
void Guntiming::write_Mode(Tango::WAttribute &attr)
{
	DEBUG_STREAM << "Guntiming::write_Mode(Tango::WAttribute &attr) entering... "<< endl;
	if (myconnected < 0) {Tango::Except::throw_exception((const char *) "NOT CONNECTED",(const char *) "NOT CONNECTED",(const char *) "Guntiming::write_FilCur()", Tango::ERR);}
	attr.get_write_value(attr_Mode_write);
	char cmd[200];
	sprintf(cmd, "ST_MODE,%3.0f\n", attr_Mode_write);
	cout << cmd << endl;
	sc.tx(cmd);
	nanosleep((const struct timespec[]){{1, 500000000L}}, NULL);
	srx = sc.rx();
	cout << "write_Mode(): " << srx << endl;
	if (srx.find("SET GU") == std::string::npos) {
		Tango::Except::throw_exception(
			(const char *) srx.c_str(),
			(const char *) srx.c_str(),
			(const char *) "Guntiming::write_Mode()", Tango::ERR);
		timestamp(); cerr << tmBuffer << "Guntiming::write_Mode() " << srx.c_str() << endl;
	}
}

//+----------------------------------------------------------------------------
//
// method :	     Guntiming::read_Mode
//
// description :	Extract real attribute values for Mode acquisition result.
//
//-----------------------------------------------------------------------------
void Guntiming::read_Mode(Tango::Attribute &attr)
{
	DEBUG_STREAM << "Guntiming::read_Mode(Tango::Attribute &attr) entering... "<< endl;
	if (myconnected < 0) {attr.set_quality(Tango::ATTR_INVALID, true); return;}
	*attr_Mode_read = std::stof(tokens[12].substr(tokens[12].find(":") + 1));
	attr.set_value(attr_Mode_read);
}

//+----------------------------------------------------------------------------
//
// method :	     Guntiming::read_Mode string
//
// description :	Extract real attribute values for Mode string acquisition result.
//
//-----------------------------------------------------------------------------
void Guntiming::read_Modestring(Tango::Attribute &attr)
{
	DEBUG_STREAM << "Guntiming::read_Modestring(Tango::Attribute &attr) entering... "<< endl;
	if (myconnected < 0) {attr.set_quality(Tango::ATTR_INVALID, true); return;}
	int i = std::stof(tokens[12].substr(tokens[12].find(":") + 1));
	if (i == 0) sprintf(tstr, "Disable");
	else if (i == 1) sprintf(tstr, "Multi bunch");
	else if (i == 2) sprintf(tstr, "Single bunch");
	else sprintf(tstr, "Invalid mode");
	*attr_Modestring_read = tstr;
	attr.set_value(attr_Modestring_read);
}

//+----------------------------------------------------------------------------
//
// method :	     Guntiming::write_FilCur
//
// description :	Write FilCur attribute values to hardware.
//
//-----------------------------------------------------------------------------
void Guntiming::write_FilCur(Tango::WAttribute &attr)
{
	DEBUG_STREAM << "Guntiming::write_FilCur(Tango::WAttribute &attr) entering... "<< endl;
	if (myconnected < 0) {Tango::Except::throw_exception((const char *) "NOT CONNECTED",(const char *) "NOT CONNECTED",(const char *) "Guntiming::write_FilCur()", Tango::ERR);}
	attr.get_write_value(attr_FilCur_write);
	char cmd[200];
	sprintf(cmd, "ST_CUR,%3.0f\n", attr_FilCur_write);
	cout << cmd << endl;
	sc.tx(cmd);
	nanosleep((const struct timespec[]){{1, 500000000L}}, NULL);
	srx = sc.rx();
	cout << "write_FilCur(): " << srx << endl;
	if (srx.find("SET Filamento to") == std::string::npos) {
		Tango::Except::throw_exception(
			(const char *) srx.c_str(),
			(const char *) srx.c_str(),
			(const char *) "Guntiming::write_FilCur()", Tango::ERR);
		timestamp(); cerr << tmBuffer << "Guntiming::write_FilCur() " << srx.c_str() << endl;
	}
}

//+----------------------------------------------------------------------------
//
// method :	     Guntiming::read_FilCur
//
// description :	Extract real attribute values for FilCur acquisition result.
//
//-----------------------------------------------------------------------------
void Guntiming::read_FilCur(Tango::Attribute &attr)
{
	DEBUG_STREAM << "Guntiming::read_FilCur(Tango::Attribute &attr) entering... "<< endl;
	if (myconnected < 0) {attr.set_quality(Tango::ATTR_INVALID, true); return;}
	*attr_FilCur_read = std::stof(tokens[7].substr(tokens[7].find(":") + 1));
	attr.set_value(attr_FilCur_read);
}

//+----------------------------------------------------------------------------
//
// method :	     Guntiming::read_FilCurset
//
// description :	Extract real attribute values for FilCurset acquisition result.
//
//-----------------------------------------------------------------------------
void Guntiming::read_FilCurset(Tango::Attribute &attr)
{
	DEBUG_STREAM << "Guntiming::read_FilCurset(Tango::Attribute &attr) entering... "<< endl;
	if (myconnected < 0) {attr.set_quality(Tango::ATTR_INVALID, true); return;}
	*attr_FilCurset_read = std::stof(tokens[6].substr(tokens[6].find(":") + 1));
	attr.set_value(attr_FilCurset_read);
}

//+----------------------------------------------------------------------------
//
// method :	     Guntiming::read_FilVoltage
//
// description :	Extract real attribute values for FilVoltage acquisition result.
//
//-----------------------------------------------------------------------------
void Guntiming::read_FilVoltage(Tango::Attribute &attr)
{
	DEBUG_STREAM << "Guntiming::read_FilVoltage(Tango::Attribute &attr) entering... "<< endl;
	if (myconnected < 0) {attr.set_quality(Tango::ATTR_INVALID, true); return;}
	*attr_FilVoltage_read = std::stof(tokens[8].substr(tokens[8].find(":") + 1));
	attr.set_value(attr_FilVoltage_read);
}

//+----------------------------------------------------------------------------
//
// method :	     Guntiming::read_GridVoltage
//
// description :	Extract real attribute values for GridVoltage acquisition result.
//
//-----------------------------------------------------------------------------
void Guntiming::read_GridVoltage(Tango::Attribute &attr)
{
	DEBUG_STREAM << "Guntiming::read_GridVoltage(Tango::Attribute &attr) entering... "<< endl;
	if (myconnected < 0) {attr.set_quality(Tango::ATTR_INVALID, true); return;}
	*attr_GridVoltage_read = std::stof(tokens[3].substr(tokens[3].find(":") + 1));
	attr.set_value(attr_GridVoltage_read);
	
}

//+----------------------------------------------------------------------------
//
// method :	     Guntiming::read_PolVoltage
//
// description :	Extract real attribute values for PolVoltage acquisition result.
//
//-----------------------------------------------------------------------------
void Guntiming::read_PolVoltage(Tango::Attribute &attr)
{
	DEBUG_STREAM << "Guntiming::read_PolVoltage(Tango::Attribute &attr) entering... "<< endl;
	if (myconnected < 0) {attr.set_quality(Tango::ATTR_INVALID, true); return;}
	*attr_PolVoltage_read = std::stof(tokens[2].substr(tokens[2].find(":") + 1));
	attr.set_value(attr_PolVoltage_read);
	
}

//+----------------------------------------------------------------------------
//
// method :	     Guntiming::read_Temperature
//
// description :	Extract real attribute values for Temperature acquisition result.
//
//-----------------------------------------------------------------------------
void Guntiming::read_Temperature(Tango::Attribute &attr)
{
	DEBUG_STREAM << "Guntiming::read_Temperature(Tango::Attribute &attr) entering... "<< endl;
	if (myconnected < 0) {attr.set_quality(Tango::ATTR_INVALID, true); return;}
	*attr_Temperature_read = std::stof(tokens[10].substr(tokens[10].find(":") + 1));
	attr.set_value(attr_Temperature_read);
	
}

//+----------------------------------------------------------------------------
//
// method :	     Guntiming::read_DriverVoltage
//
// description :	Extract real attribute values for DriverVoltage acquisition result.
//
//-----------------------------------------------------------------------------
void Guntiming::read_DriverVoltage(Tango::Attribute &attr)
{
	DEBUG_STREAM << "Guntiming::read_DriverVoltage(Tango::Attribute &attr) entering... "<< endl;
	if (myconnected < 0) {attr.set_quality(Tango::ATTR_INVALID, true); return;}
	*attr_DriverVoltage_read = std::stof(tokens[4].substr(tokens[4].find(":") + 1));
	attr.set_value(attr_DriverVoltage_read);
	
}

//+----------------------------------------------------------------------------
//
// method :	     Guntiming::read_DriverCur
//
// description :	Extract real attribute values for DriverCur acquisition result.
//
//-----------------------------------------------------------------------------
void Guntiming::read_DriverCur(Tango::Attribute &attr)
{
	DEBUG_STREAM << "Guntiming::read_DriverCur(Tango::Attribute &attr) entering... "<< endl;
	if (myconnected < 0) {attr.set_quality(Tango::ATTR_INVALID, true); return;}
	*attr_DriverCur_read = std::stof(tokens[5].substr(tokens[5].find(":") + 1));
	attr.set_value(attr_DriverCur_read);
	
}

//+----------------------------------------------------------------------------
//
// method :	     Guntiming::read_BunchLength
//
// description :	Extract real attribute values for BunchLength acquisition result.
//
//-----------------------------------------------------------------------------
void Guntiming::read_BunchLength(Tango::Attribute &attr)
{
	DEBUG_STREAM << "Guntiming::read_BunchLength(Tango::Attribute &attr) entering... "<< endl;
	if (myconnected < 0) {attr.set_quality(Tango::ATTR_INVALID, true); return;}
	*attr_Mode_read = std::stof(tokens[11].substr(tokens[11].find(":") + 1)) * 2;
	attr.set_value(attr_Mode_read);
}

//+----------------------------------------------------------------------------
//
// method :	     Guntiming::write_BunchLength
//
// description :	Write BunchLength attribute values to hardware.
//
//-----------------------------------------------------------------------------
void Guntiming::write_BunchLength(Tango::WAttribute &attr)
{
	DEBUG_STREAM << "Guntiming::write_BunchLength(Tango::WAttribute &attr) entering... "<< endl;
	if (myconnected < 0) {Tango::Except::throw_exception((const char *) "NOT CONNECTED",(const char *) "NOT CONNECTED",(const char *) "Guntiming::write_FilCur()", Tango::ERR);}
	attr.get_write_value(attr_FilCur_write);
	char cmd[200];
	sprintf(cmd, "ST_NB,%3.0f\n", round(attr_FilCur_write/2));
	cout << cmd << endl;
	sc.tx(cmd);
	nanosleep((const struct timespec[]){{1, 500000000L}}, NULL);
	srx = sc.rx();
	cout << "write_BunchLength(): " << srx << endl;
	if (srx.find("SET num bucket to") == std::string::npos) {
		Tango::Except::throw_exception(
			(const char *) cmd,
			(const char *) srx.c_str(),
			(const char *) "Guntiming::write_BunchLength()", Tango::ERR);
		timestamp(); cerr << tmBuffer << "Guntiming::write_BunchLength() " << srx.c_str() << endl;
	}
}

//+------------------------------------------------------------------
/**
 *      method: Guntiming::thy_mea_on
 *
 *      description:    method to execute raw command
 *      Switch ON thyratron/Measure signals
 *
 *
 */
//+------------------------------------------------------------------
Tango::DevVarStringArray *Guntiming::raw(Tango::DevString argin)
{
	DEBUG_STREAM << "Guntiming::raw(): entering... !" << endl;

	//      Add your own code to control device here
	if (myconnected < 0) {Tango::Except::throw_exception((const char *) "NOT CONNECTED",(const char *) "NOT CONNECTED",(const char *) "Guntiming::write_FilCur()", Tango::ERR);}
	cout << argin << endl;
	
	Tango::DevVarStringArray *argout = new Tango::DevVarStringArray();
	argout->length(1);
	sc.tx(argin);
	sc.tx("\n");
	nanosleep((const struct timespec[]){{0, 500000000L}}, NULL);
	srx = sc.rx();
	// strcpy(buf, srx.c_str());
	(*argout)[0] = CORBA::string_dup(srx.c_str());
	return argout;
	// return "test";
}

//+------------------------------------------------------------------
/**
 *      method: Guntiming::stinit
 *
 *      description:    method to execute "ST_INIT"
 *
 *
 */
//+------------------------------------------------------------------
void Guntiming::stinit()
{
	DEBUG_STREAM << "Guntiming::stinit(): entering... !" << endl;
	if (myconnected < 0) {Tango::Except::throw_exception((const char *) "NOT CONNECTED",(const char *) "NOT CONNECTED",(const char *) "Guntiming::write_FilCur()", Tango::ERR);}
	char cmd[200];
	sprintf(cmd, "ST_INIT\n");
	cout << cmd << endl;
	sc.tx(cmd);
	nanosleep((const struct timespec[]){{1, 500000000L}}, NULL);
	srx = sc.rx();
	cout << "st_init(): " << srx << endl;
	if (srx.find("INIT done") == std::string::npos) {
		Tango::Except::throw_exception(
			(const char *) cmd,
			(const char *) srx.c_str(),
			(const char *) "Guntiming::stinit()", Tango::ERR);
		timestamp(); cerr << tmBuffer << "Guntiming::write_stinit() " << srx.c_str() << endl;
	}
}

//+------------------------------------------------------------------
/**
 *      method: Guntiming::restart
 *
 *      description:    method to execute "Restart"
 *      Reset Lantronix via cURL
 *
 *
 */
//+------------------------------------------------------------------
void Guntiming::restart()
{
	DEBUG_STREAM << "Guntiming::restart(): entering... !" << endl;

	//      Add your own code to control device here
	std::string cmd = "curl \"http://"+IP+"/secure/setup.cgi\" -X POST -H \"DNT: 1\" -H \"Authorization: Basic Og==\" --data-raw \"rec0=wKjNfgAACC10EwAAwKjNAUwJAAARJwAAAAAAAMAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAEwCAAASJwAAAAAAAMAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA&loc=applyset.htm\"";
	system(cmd.c_str());
	sleep(3);
	cout << "connecting IP: " << IP << endl;
	timestamp(); cerr << tmBuffer << "connecting IP:  " << IP << endl;
	myconnect(sc.getSocket(), IP, 10001);
}

}       //      namespace

